use crate::sensor_names::CPU_NAMES;
use crate::sensor_names::DISK_NAMES;
use crate::sensor_names::TEMP_NAMES;
use anyhow::anyhow;
use anyhow::Result;
use core::f64;
use serde::Deserialize;
use std::collections::BTreeMap;
use std::process::Command;

#[derive(Debug)]
pub struct SensorData {
    chips: BTreeMap<String, Sensor>,
}

impl SensorData {
    pub fn cpu_chip(&self) -> Option<&Sensor> {
        for cpu_name in CPU_NAMES {
            if let Some(chip) = self.chips.get(cpu_name) {
                return Some(chip);
            }
        }
        None
    }

    pub fn disk_chip(&self) -> Option<&Sensor> {
        for disk_name in DISK_NAMES {
            if let Some(chip) = self.chips.get(disk_name) {
                return Some(chip);
            }
        }
        None
    }
}

#[derive(Debug, Deserialize)]
pub struct Sensor {
    #[serde(rename = "Adapter")]
    _adapter: String,

    #[serde(rename = "Tctl")]
    tctl: Option<BTreeMap<String, f64>>,

    #[serde(rename = "Tccd1")]
    tccd1: Option<BTreeMap<String, f64>>,
    temp1: Option<BTreeMap<String, f64>>,

    #[serde(rename = "Composite")]
    composite: Option<BTreeMap<String, f64>>,
}

impl Sensor {
    pub fn temp(&self) -> Option<f64> {
        let sensor = self.actual_sector();
        match sensor {
            None => return None,
            Some(temperatures) => {
                for temp_name in TEMP_NAMES {
                    if let Some(temp) = temperatures.get(temp_name) {
                        return Some(*temp);
                    }
                }
            }
        }
        None
    }

    fn actual_sector(&self) -> Option<BTreeMap<String, f64>> {
        if self.tctl.is_some() {
            return self.tctl.clone();
        }

        if self.tccd1.is_some() {
            return self.tccd1.clone();
        }

        if self.temp1.is_some() {
            return self.temp1.clone();
        }

        if self.composite.is_some() {
            return self.composite.clone();
        }

        None
    }
}

pub fn sensors_data() -> Result<SensorData> {
    let output = Command::new("sensors").arg("-j").output()?;

    if output.status.success() {
        let json_output = String::from_utf8_lossy(&output.stdout).to_string();
        let sensors_data: Result<BTreeMap<String, Sensor>, serde_json::Error> =
            serde_json::from_str(&json_output);
        match sensors_data {
            Ok(chips) => Ok(SensorData { chips }),
            Err(err) => Err(anyhow!(err)),
        }
    } else {
        Err(anyhow!(String::from_utf8_lossy(&output.stderr).to_string()))
    }
}
